function [spikes]= Andrzejak_PRE_90_062906_GenerateSpikes(systemtype,RequiredSpikeCount,verbose)
% This source code forms part of the resources provided at http://ntsa.upf.edu/downloads .
% It belongs to the library used for the computations underlying the manuscript:
% Andrzejak RG, Mormann F, Kreuz T.  2014.  Detecting determinism from point processes. Phys. Rev. E. 90, 062906
% In order to understand how the source code works, you should at first
% read the manuscript. A preprint can be found following the link above.
% If you use the source codes, please make sure that you give a reference to
% the manuscript. The correct way to cite this paper is shown above.
% No page numbers should be given in the reference.
%
% +++ What this particular script does:
%
% This script generates the script for the models A-B
% It will always return exactly a number of RequiredSpikeCount.
% The first spike is a time zero and the last spike is at time 1
%
% ++++ Input ++++
% Systemtype
%   case 1: model A
%   case 2: model B
%   case 3: model C
%   case 4: model D
%   case 5: model E
%
% RequiredSpikeCount: As the name suggests.
%
% verbose:
%  case 0: no screen output
%  case 1: some screen output
%
% ++++ Output ++++
% the spike train


spikecount = 0;
timeinterval = 1;
% In the following while-loop, we set
% a time step for the integration dt,
% and a number of data points Q2 from which we detect the spikes.
% In each case, Q2 is set accordingly to the time scale of each dynamics.
% If the resulting spikecount is less than RequiredSpikeCount, we try again
% with Q2 twice as high, until we have enough spikes.
% The first 20% of the time is always discarded to let transients die out.
while spikecount<RequiredSpikeCount
    switch systemtype
        case 1
            if (verbose && (timeinterval==1))
                fprintf('Using model A\n');
            end
            dt = 0.005;
            Q2 = round(50000*1.1*timeinterval);
            preiterations = round(Q2*0.2);
            x = Andrzejak_PRE_90_062906_RungeKuttaIntegrator(@Andrzejak_PRE_90_062906_LorenzODE,Q2+preiterations,dt,rand(3,1));
            [spikes, spikecount] = Andrzejak_PRE_90_062906_DetectSpikesIAF(x(1,preiterations+1:Q2+preiterations),dt,12);
        case 2
            if (verbose && (timeinterval==1))
                fprintf('Using model B\n');
            end
            dt = 0.005;
            Q2 = round(50000*1.8*timeinterval);
            preiterations = round(Q2*0.2);
            x = Andrzejak_PRE_90_062906_RungeKuttaIntegrator(@Andrzejak_PRE_90_062906_LorenzODE,Q2+preiterations,dt,rand(3,1));
            [spikes, spikecount] = Andrzejak_PRE_90_062906_DetectSpikesThreshold(x(3,preiterations+1:Q2+preiterations),27);
        case 3
            if (verbose && (timeinterval==1))
                fprintf('Using model C\n');
            end
            dt = 0.1;
            Q2 = round(50000*5*timeinterval);
            preiterations = round(Q2*0.2);
            x = Andrzejak_PRE_90_062906_RungeKuttaIntegrator(@Andrzejak_PRE_90_062906_HindMarshRoseJ33ODE,Q2+preiterations,dt,rand(3,1));
            [spikes, spikecount] = Andrzejak_PRE_90_062906_DetectSpikesThreshold(x(1,preiterations+1:Q2+preiterations),0.6);
        case 4
            if (verbose && (timeinterval==1))
                fprintf('Using model D\n');
            end
            dt = 0.1;
            Q2 = round(50000*4*timeinterval);
            preiterations = round(Q2*0.2);
            x = Andrzejak_PRE_90_062906_RungeKuttaIntegrator(@Andrzejak_PRE_90_062906_HindMarshRoseJ328ODE,Q2+preiterations,dt,rand(3,1));
            [spikes, spikecount] = Andrzejak_PRE_90_062906_DetectSpikesThreshold(x(1,preiterations+1:Q2+preiterations),0.6);
        case 5
            if (verbose && (timeinterval==1))
                fprintf('Using model E\n');
            end
            
            dt = 0.005;
            Q2 = round(50000*1.1*timeinterval);
            preiterations = round(Q2*0.2);
            x = Andrzejak_PRE_90_062906_RungeKuttaIntegrator(@Andrzejak_PRE_90_062906_LorenzODE,Q2+preiterations,dt,rand(3,1));
            [spikes, spikecount] = Andrzejak_PRE_90_062906_DetectSpikesIAF(Andrzejak_PRE_90_062906_RandomizePhases(x(1,preiterations+1:Q2+preiterations)),dt,12);
    end
    timeinterval = timeinterval*2;
end

spikes = spikes(1:RequiredSpikeCount);
spikes = spikes-min(spikes);    % first spike will be at zero
spikes = spikes/max(spikes);    % last spikes will be at 1

